/*:
 * @target MZ
 * @plugindesc [v1.8] Умная система аудио – fade, pause/resume, запуск с указанного момента (startfrom), запуск с fade (startwithfade), сброс всех аудио и регулировка volume для BGM/BGS/SE/ME.
 * @author Sunohara
 *
 * @help
 * Этот плагин расширяет стандартные возможности работы со звуком:
 *  • Плавное появление (fade-in) и затухание (fade-out) для BGM, BGS, SE и ME.
 *  • Остановка (pause) с сохранением позиции и последующее возобновление (resume).
 *     - Если пытаемся вызвать resume, когда музыкальный файл уже играет, выводится ошибка:
 *         "Нельзя воспроизвести уже играющий музыкальный файл типа: <тип>."
 *     - Если пытаемся вызвать pause, когда музыка не играет, выводится ошибка:
 *         "В данный момент не играет музыкальный файл типа: <тип>."
 *  • Запуск трека с указанного момента (startfrom) и запуск с fade-in (startwithfade).
 *     При переключении музыки предыдущая композиция забывается – сохраняется только последняя запущенная.
 *  • Сброс всех аудио (resetall): останавливаются BGM, BGS, ME и SE, а также очищаются их сохранённые состояния.
 *     Если музыка не играет, выводится ошибка:
 *         "В данный момент не играет ни один музыкальный файл для сброса!"
 *  • Регулировка volume для всех команд:
 *     - Дополнительный параметр volume (например, resume(bgm, 5, 50)) позволяет задать уровень громкости (50) при возобновлении.
 *
 * Готовые примеры реализации для игровых событий:
 *
 * =============================================================================
 *
 * Воспроизвести BGM "Battle1" с первой минуты (при громкости 90 и плавности в пять секунд)
 *   SmartAudio.startfrom(bgm, "Battle1", "1:00", 90, 0, 0, 5);
 * Воспроизвести BGS "Town" с тридцатой секунды (при громкости 80 и плавности в две секунды)
 *   SmartAudio.startfrom(bgs, "Town", "0:30", 80, 100, 0, 2);
 * Воспроизвести SE "Explosion" с плавностью в одну секунду и при громкости 70 (здесь другой порядок аргументов) 
 *   SmartAudio.startwithfade(se, "Explosion", 1, 70, 100, 0);
 * Поставить на паузу дорожку типа BGM с плавным затуханием в три секунды
 *   SmartAudio.pause(bgm, 3);
 * Воспроизвести дорожку BGM с последнего места с плавностью в пять секунд при громкости 80
 *   SmartAudio.resume(bgm, 5, 80);
 * Очистить все музыкальные каналы (BGM, BGS, ME и SE)
 *   SmartAudio.resetall();
 *
 * Глобальные переменные:
 *   bgm = "bgm", bgs = "bgs", se = "se", me = "me"
 *
 * =============================================================================
 *
 * Plugin Commands (аргументы в кавычках) работают как обычно.
 */

(function() {
    "use strict";
    
    // ======================================================
    // Расширение Game_System для хранения состояния аудио
    // ======================================================
    const _Game_System_initialize = Game_System.prototype.initialize;
    Game_System.prototype.initialize = function() {
        _Game_System_initialize.call(this);
        // Объект для хранения состояния аудио по типам (например, bgm, bgs, me, se)
        this._smartAudioState = {};
    };

    // ======================================================
    // Глобальные переменные для типов аудио
    // ======================================================
    window.bgm = "bgm";
    window.bgs = "bgs";
    window.se  = "se";
    window.me  = "me";

    // ======================================================
    // Основной объект плагина SmartAudio
    // ======================================================
    window.SmartAudio = {};

    // ------------------------------------------------------
    // Функция: parseTimeString
    // Преобразует строку времени "m:ss" в секунды.
    // ------------------------------------------------------
    SmartAudio.parseTimeString = function(timeStr) {
        timeStr = String(timeStr);
        const parts = timeStr.split(':');
        if (parts.length !== 2) return 0;
        const minutes = parseInt(parts[0], 10);
        const seconds = parseInt(parts[1], 10);
        if (isNaN(minutes) || isNaN(seconds)) return 0;
        return minutes * 60 + seconds;
    };

    // ------------------------------------------------------
    // Функция: fadeIn
    // Плавное увеличение громкости аудио.
    // buffer      – объект звука (например, AudioManager._bgmBuffer)
    // targetVolume– конечная громкость (от 0 до 1)
    // duration    – время в секундах
    // ------------------------------------------------------
    SmartAudio.fadeIn = function(buffer, targetVolume, duration) {
        if (!buffer) return;
        const startTime = performance.now();
        const initialVolume = 0;
        buffer.volume = initialVolume;
        const animate = function() {
            const elapsed = (performance.now() - startTime) / 1000;
            const ratio = Math.min(elapsed / duration, 1);
            buffer.volume = initialVolume + (targetVolume - initialVolume) * ratio;
            if (ratio < 1) {
                requestAnimationFrame(animate);
            }
        };
        animate();
    };

    // ------------------------------------------------------
    // Функция: fadeOut
    // Плавное затухание аудио.
    // После завершения fade out трек останавливается.
    // ------------------------------------------------------
    SmartAudio.fadeOut = function(buffer, duration) {
        if (!buffer) return;
        const startTime = performance.now();
        const initialVolume = buffer.volume;
        const animate = function() {
            const elapsed = (performance.now() - startTime) / 1000;
            const ratio = Math.min(elapsed / duration, 1);
            buffer.volume = initialVolume * (1 - ratio);
            if (ratio < 1) {
                requestAnimationFrame(animate);
            } else {
                buffer.stop();
            }
        };
        animate();
    };

    // ------------------------------------------------------
    // Функция: startfrom
    // Запускает аудиотрек указанного типа с заданного момента.
    // Обязательные параметры:
    //   type    – тип аудио ("bgm", "bgs", "se" или "me")
    //   name    – имя файла (без расширения)
    //   timeStr – строка времени ("m:ss", напр. "1:00")
    // Опционально:
    //   volume  – громкость (по умолчанию 90)
    //   pitch   – высота тона (по умолчанию 100)
    //   pan     – панорама (по умолчанию 0)
    //   fadeIn  – время плавного входа (по умолчанию 0)
    // ------------------------------------------------------
    SmartAudio.startfrom = function(type, name, timeStr, volume, pitch, pan, fadeIn) {
        type    = String(type).toLowerCase();
        name    = String(name);
        timeStr = String(timeStr);
        if (typeof volume === "undefined") volume = 90;
        if (typeof pitch  === "undefined") pitch  = 100;
        if (typeof pan    === "undefined") pan    = 0;
        if (typeof fadeIn === "undefined") fadeIn = 0;
        const pos = SmartAudio.parseTimeString(timeStr);
        const audio = { name: name, volume: volume, pitch: pitch, pan: pan };

        // При переключении трека забываем старые данные
        delete $gameSystem._smartAudioState[type];

        switch(type) {
            case 'bgm':
                AudioManager.stopBgm();
                AudioManager.playBgm(audio, pos);
                if (AudioManager._bgmBuffer) {
                    if (fadeIn > 0) {
                        AudioManager._bgmBuffer.volume = 0;
                        SmartAudio.fadeIn(AudioManager._bgmBuffer, volume / 100, fadeIn);
                    } else {
                        AudioManager._bgmBuffer.volume = volume / 100;
                    }
                }
                $gameSystem._smartAudioState.bgm = { audio: audio, pos: pos };
                break;
            case 'bgs':
                AudioManager.stopBgs();
                AudioManager.playBgs(audio, pos);
                if (AudioManager._bgsBuffer) {
                    if (fadeIn > 0) {
                        AudioManager._bgsBuffer.volume = 0;
                        SmartAudio.fadeIn(AudioManager._bgsBuffer, volume / 100, fadeIn);
                    } else {
                        AudioManager._bgsBuffer.volume = volume / 100;
                    }
                }
                $gameSystem._smartAudioState.bgs = { audio: audio, pos: pos };
                break;
            case 'se':
                AudioManager.playSe(audio);
                $gameSystem._smartAudioState.se = { audio: audio, pos: 0 };
                break;
            case 'me':
                AudioManager.stopMe();
                AudioManager.playMe(audio);
                if (AudioManager._meBuffer) {
                    AudioManager._meBuffer.volume = volume / 100;
                }
                $gameSystem._smartAudioState.me = { audio: audio, pos: 0 };
                break;
            default:
                console.warn("SmartAudio: неизвестный тип аудио: " + type);
                break;
        }
    };

    // ------------------------------------------------------
    // Функция: startwithfade
    // Запускает аудиотрек указанного типа с позиции 0 с применением fade‑in.
    // Обязательные параметры:
    //   type         – тип аудио ("bgm", "bgs", "se" или "me")
    //   song         – имя файла (без расширения)
    //   fadeDuration – длительность fade‑in (секунды)
    // Опционально:
    //   volume  – громкость (по умолчанию 90)
    //   pitch   – высота тона (по умолчанию 100)
    //   pan     – панорама (по умолчанию 0)
    // ------------------------------------------------------
    SmartAudio.startwithfade = function(type, song, fadeDuration, volume, pitch, pan) {
        type         = String(type).toLowerCase();
        song         = String(song);
        fadeDuration = Number(fadeDuration);
        if (typeof volume === "undefined") volume = 90;
        if (typeof pitch === "undefined") pitch = 100;
        if (typeof pan === "undefined") pan = 0;
        const pos    = 0;
        const audio  = { name: song, volume: volume, pitch: pitch, pan: pan };

        // При переключении трека забываем старые данные
        delete $gameSystem._smartAudioState[type];

        switch(type) {
            case 'bgm':
                AudioManager.stopBgm();
                AudioManager.playBgm(audio, pos);
                if (AudioManager._bgmBuffer) {
                    if (fadeDuration > 0) {
                        AudioManager._bgmBuffer.volume = 0;
                        SmartAudio.fadeIn(AudioManager._bgmBuffer, volume / 100, fadeDuration);
                    } else {
                        AudioManager._bgmBuffer.volume = volume / 100;
                    }
                }
                $gameSystem._smartAudioState.bgm = { audio: audio, pos: pos };
                break;
            case 'bgs':
                AudioManager.stopBgs();
                AudioManager.playBgs(audio, pos);
                if (AudioManager._bgsBuffer) {
                    if (fadeDuration > 0) {
                        AudioManager._bgsBuffer.volume = 0;
                        SmartAudio.fadeIn(AudioManager._bgsBuffer, volume / 100, fadeDuration);
                    } else {
                        AudioManager._bgsBuffer.volume = volume / 100;
                    }
                }
                $gameSystem._smartAudioState.bgs = { audio: audio, pos: pos };
                break;
            case 'se':
                AudioManager.playSe(audio);
                $gameSystem._smartAudioState.se = { audio: audio, pos: 0 };
                break;
            case 'me':
                AudioManager.stopMe();
                AudioManager.playMe(audio);
                if (AudioManager._meBuffer) {
                    if (fadeDuration > 0) {
                        AudioManager._meBuffer.volume = 0;
                        SmartAudio.fadeIn(AudioManager._meBuffer, volume / 100, fadeDuration);
                    } else {
                        AudioManager._meBuffer.volume = volume / 100;
                    }
                }
                $gameSystem._smartAudioState.me = { audio: audio, pos: pos };
                break;
            default:
                console.warn("SmartAudio: неизвестный тип аудио: " + type);
                break;
        }
    };

    // ------------------------------------------------------
    // Функция: pause
    // Сохраняет текущую позицию аудио и останавливает воспроизведение.
    // Если задан fadeOutTime – применяется плавное затухание.
    // Если аудио не играет, выводится ошибка.
    // ------------------------------------------------------
    SmartAudio.pause = function(type, fadeOutTime = 0) {
        type = String(type).toLowerCase();
        fadeOutTime = Number(fadeOutTime);
        let buffer, stopFunc, stateKey;
        switch(type) {
            case 'bgm':
                buffer = AudioManager._bgmBuffer;
                stopFunc = () => AudioManager.stopBgm();
                stateKey = "bgm";
                break;
            case 'bgs':
                buffer = AudioManager._bgsBuffer;
                stopFunc = () => AudioManager.stopBgs();
                stateKey = "bgs";
                break;
            case 'me':
                buffer = AudioManager._meBuffer;
                stopFunc = () => AudioManager.stopMe();
                stateKey = "me";
                break;
            case 'se':
                console.warn("SmartAudio: функция pause не поддерживается для SE.");
                return;
            default:
                console.warn("SmartAudio: неизвестный тип аудио: " + type);
                return;
        }
        if (!buffer || !buffer.isPlaying || !buffer.isPlaying()) {
            console.error("В данный момент не играет музыкальный файл типа: " + type + ".");
            return;
        }
        const currentPos = buffer.seek();
        let currentAudio;
        switch(type) {
            case 'bgm':
                currentAudio = AudioManager._currentBgm;
                break;
            case 'bgs':
                currentAudio = AudioManager._currentBgs;
                break;
            case 'me':
                currentAudio = AudioManager._currentMe;
                break;
        }
        if (fadeOutTime > 0 && buffer) {
            SmartAudio.fadeOut(buffer, fadeOutTime);
        } else {
            stopFunc();
        }
        $gameSystem._smartAudioState[stateKey] = { audio: currentAudio, pos: currentPos };
    };

    // ------------------------------------------------------
    // Функция: resume
    // Возобновляет воспроизведение ранее остановленного аудио с сохранённой позиции.
    // Если аудио уже играет или состояние отсутствует, выводится ошибка.
    // Для BGM, BGS и ME применяется небольшой таймаут для ожидания загрузки буфера.
    // Дополнительно можно задать уровень громкости (volume), который переопределит сохранённое значение.
    // ------------------------------------------------------
    SmartAudio.resume = function(type, fadeInTime = 0, volumeOverride) {
        type = String(type).toLowerCase();
        fadeInTime = Number(fadeInTime);
        if (!$gameSystem._smartAudioState.hasOwnProperty(type)) {
            if (SmartAudio.isPlaying(type)) {
                console.error("Нельзя воспроизвести уже играющий музыкальный файл типа: " + type + ".");
            } else {
                console.error("Нет приостановленного аудио типа: " + type + ".");
            }
            return;
        }
        if (SmartAudio.isPlaying(type)) {
            console.error("Нельзя воспроизвести уже играющий музыкальный файл типа: " + type + ".");
            return;
        }
        const state = $gameSystem._smartAudioState[type];
        // Если задано переопределение громкости, обновляем состояние
        if (typeof volumeOverride !== "undefined") {
            state.audio.volume = Number(volumeOverride);
        }
        switch(type) {
            case 'bgm':
                AudioManager.stopBgm();
                AudioManager.playBgm(state.audio, state.pos);
                if (AudioManager._bgmBuffer) {
                    setTimeout(function() {
                        if (fadeInTime > 0) {
                            AudioManager._bgmBuffer.volume = 0;
                            SmartAudio.fadeIn(AudioManager._bgmBuffer, state.audio.volume / 100, fadeInTime);
                        } else {
                            AudioManager._bgmBuffer.volume = state.audio.volume / 100;
                        }
                    }, 100);
                }
                break;
            case 'bgs':
                AudioManager.stopBgs();
                AudioManager.playBgs(state.audio, state.pos);
                if (AudioManager._bgsBuffer) {
                    setTimeout(function() {
                        if (fadeInTime > 0) {
                            AudioManager._bgsBuffer.volume = 0;
                            SmartAudio.fadeIn(AudioManager._bgsBuffer, state.audio.volume / 100, fadeInTime);
                        } else {
                            AudioManager._bgsBuffer.volume = state.audio.volume / 100;
                        }
                    }, 100);
                }
                break;
            case 'me':
                AudioManager.stopMe();
                AudioManager.playMe(state.audio);
                if (AudioManager._meBuffer) {
                    setTimeout(function() {
                        if (AudioManager._meBuffer._source) {
                            try {
                                AudioManager._meBuffer._source.currentTime = state.pos;
                            } catch (e) {
                                console.warn("SmartAudio.resume: не удалось установить позицию ME: " + e);
                            }
                        }
                        if (fadeInTime > 0) {
                            AudioManager._meBuffer.volume = 0;
                            SmartAudio.fadeIn(AudioManager._meBuffer, state.audio.volume / 100, fadeInTime);
                        } else {
                            AudioManager._meBuffer.volume = state.audio.volume / 100;
                        }
                    }, 100);
                }
                break;
            case 'se':
                console.warn("SmartAudio: функция resume не поддерживается для SE.");
                return;
            default:
                console.warn("SmartAudio: неизвестный тип аудио: " + type);
                return;
        }
        delete $gameSystem._smartAudioState[type];
    };

    // ------------------------------------------------------
    // Функция: isPlaying
    // Проверяет, воспроизводится ли аудио данного типа.
    // ------------------------------------------------------
    SmartAudio.isPlaying = function(type) {
        type = String(type).toLowerCase();
        let buffer;
        switch(type) {
            case 'bgm': buffer = AudioManager._bgmBuffer; break;
            case 'bgs': buffer = AudioManager._bgsBuffer; break;
            case 'me':  buffer = AudioManager._meBuffer; break;
            case 'se':  
                buffer = AudioManager._seBuffer;
                break;
            default: return false;
        }
        return buffer && typeof buffer.isPlaying === "function" && buffer.isPlaying();
    };

    // ------------------------------------------------------
    // Функция: resetall
    // Сбрасывает (останавливает) все аудио дорожки (BGM, BGS, ME и SE) и очищает их сохранённые состояния.
    // Если ни один музыкальный файл не играет, выводится ошибка в консоль.
    // ------------------------------------------------------
    SmartAudio.resetall = function() {
        const types = ["bgm", "bgs", "me", "se"];
        let anyPlaying = false;
        for (let t of types) {
            if (t === "se") {
                if ($gameSystem._smartAudioState.hasOwnProperty(t)) {
                    anyPlaying = true;
                    break;
                }
            } else {
                if (SmartAudio.isPlaying(t) || $gameSystem._smartAudioState.hasOwnProperty(t)) {
                    anyPlaying = true;
                    break;
                }
            }
        }
        if (!anyPlaying) {
            console.error("В данный момент не играет ни один музыкальный файл для сброса!");
            return;
        }
        AudioManager.stopBgm();
        AudioManager.stopBgs();
        AudioManager.stopMe();
        if (typeof AudioManager.stopSe === "function") {
            AudioManager.stopSe();
        }
        $gameSystem._smartAudioState = {};
    };

    // ======================================================
    // Регистрация Plugin Command для RPG Maker MZ
    // (аргументы в плагин-командах остаются в кавычках)
    // ======================================================
    
    PluginManager.registerCommand('SmartAudio', 'pause', args => {
        SmartAudio.pause(args.type, Number(args.fadeOut || 0));
    });

    PluginManager.registerCommand('SmartAudio', 'resume', args => {
        SmartAudio.resume(args.type, Number(args.fadeIn || 0), Number(args.volume));
    });

    PluginManager.registerCommand('SmartAudio', 'startfrom', args => {
        SmartAudio.startfrom(
            args.type,
            args.name,
            args.time,
            Number(args.volume || 90),
            Number(args.pitch || 100),
            Number(args.pan || 0),
            Number(args.fadeIn || 0)
        );
    });

    PluginManager.registerCommand('SmartAudio', 'startwithfade', args => {
        SmartAudio.startwithfade(args.type, args.song, Number(args.fadeDuration || 0), Number(args.volume || 90), Number(args.pitch || 100), Number(args.pan || 0));
    });
    
    PluginManager.registerCommand('SmartAudio', 'resetall', args => {
        SmartAudio.resetall();
    });

})();
